@@ -1,4 +1,4 @@
-/* Copyright (C) 2013-2019 Greenbone Networks GmbH
+/* Copyright (C) 2013-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -95,6 +95,7 @@ struct gvm_hosts
   size_t current;     /**< Current host index in iteration. */
   size_t count;       /**< Number of single host objects in hosts list. */
   size_t removed;     /**< Number of duplicate/excluded values. */
+  size_t duplicated;  /**< Number of duplicated values. */
 };
 
 /* Function prototypes. */
@@ -110,6 +111,9 @@ gvm_hosts_new_with_max (const gchar *, unsigned int);
 gvm_host_t *
 gvm_hosts_next (gvm_hosts_t *);
 
+void
+gvm_hosts_move_current_host_to_end (gvm_hosts_t *);
+
 void
 gvm_hosts_free (gvm_hosts_t *);
 
@@ -149,6 +153,9 @@ gvm_hosts_count (const gvm_hosts_t *);
 unsigned int
 gvm_hosts_removed (const gvm_hosts_t *);
 
+unsigned int
+gvm_hosts_duplicated (const gvm_hosts_t *);
+
 /* gvm_host_t related */
 
 gvm_host_t *
@@ -158,6 +165,10 @@ int
 gvm_host_in_hosts (const gvm_host_t *, const struct in6_addr *,
                    const gvm_hosts_t *);
 
+gvm_host_t *
+gvm_host_find_in_hosts (const gvm_host_t *, const struct in6_addr *,
+                        const gvm_hosts_t *);
+
 gchar *
 gvm_host_type_str (const gvm_host_t *);
 
@@ -176,6 +187,13 @@ gvm_host_get_addr6 (const gvm_host_t *, struct in6_addr *);
 void
 gvm_host_add_reverse_lookup (gvm_host_t *);
 
+void gvm_host_free (gpointer);
+
+gpointer gvm_duplicate_vhost (gconstpointer, gpointer);
+
+gvm_host_t *
+gvm_duplicate_host (gvm_host_t *);
+
 /* Miscellaneous functions */
 
 gvm_vhost_t *
