@@ -1,4 +1,4 @@
-/* Copyright (C) 2020 Greenbone Networks GmbH
+/* Copyright (C) 2020-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -28,9 +28,16 @@
 gvm_host_t *
 get_host_from_queue (kb_t, gboolean *);
 
+void
+put_host_on_queue (kb_t, char *);
+
 void
 put_finish_signal_on_queue (void *);
 
+void realloc_finish_signal_on_queue (kb_t);
+
+int finish_signal_on_queue (kb_t);
+
 void
 send_dead_hosts_to_ospd_openvas (int);
 
@@ -46,4 +53,7 @@ get_openvas_scan_id (const gchar *, int);
 boreas_error_t
 get_alive_test_methods (alive_test_t *);
 
+const gchar *
+get_alive_test_ports (void);
+
 #endif /* not BOREAS_IO_H */
