@@ -1,4 +1,4 @@
-/* Copyright (C) 2020 Greenbone Networks GmbH
+/* Copyright (C) 2020-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -24,11 +24,13 @@
 #include <errno.h>
 #include <glib.h>
 #include <ifaddrs.h> /* for getifaddrs() */
+#include <linux/sockios.h>
 #include <net/ethernet.h>
 #include <net/if.h>           /* for if_nametoindex() */
 #include <netpacket/packet.h> /* for sockaddr_ll */
 #include <stdlib.h>
 #include <string.h>
+#include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <unistd.h>
 
@@ -36,7 +38,7 @@
 /**
  * @brief GLib log domain.
  */
-#define G_LOG_DOMAIN "alive scan"
+#define G_LOG_DOMAIN "libgvm boreas"
 
 static boreas_error_t
 set_socket (socket_type_t, int *);
@@ -287,7 +289,9 @@ fill_ports_array (gpointer range, gpointer ports_array)
   gboolean range_exclude;
   int range_start;
   int range_end;
-  int port;
+  int i;
+  /* Use uint16_t for port array elements. tcphdr port type is uint16_t. */
+  uint16_t port_sized;
 
   range_start = ((range_t *) range)->start;
   range_end = ((range_t *) range)->end;
@@ -300,13 +304,17 @@ fill_ports_array (gpointer range, gpointer ports_array)
   /* Only single port in range. */
   if (range_end == 0 || (range_start == range_end))
     {
-      g_array_append_val (ports_array, range_start);
+      port_sized = (uint16_t) range_start;
+      g_array_append_val (ports_array, port_sized);
       return;
     }
   else
     {
-      for (port = range_start; port <= range_end; port++)
-        g_array_append_val (ports_array, port);
+      for (i = range_start; i <= range_end; i++)
+        {
+          port_sized = (uint16_t) i;
+          g_array_append_val (ports_array, port_sized);
+        }
     }
 }
 
@@ -409,7 +417,7 @@ static boreas_error_t
 set_socket (socket_type_t socket_type, int *scanner_socket)
 {
   boreas_error_t error = NO_ERROR;
-  int soc;
+  int soc = -1;
   switch (socket_type)
     {
     case UDPV4:
@@ -607,3 +615,45 @@ count_difference (GHashTable *hashtable_A, GHashTable *hashtable_B)
 
   return count;
 }
+
+/**
+ * @brief Check if socket send buffer is empty.
+ *
+ * @param[in] soc Socket.
+ * @param[out] err Set to -1 on error.
+ *
+ * @return 1 if so_sndbug is empyt, else 0.
+ */
+static int
+so_sndbuf_empty (int soc, int *err)
+{
+  int cur_so_sendbuf = -1;
+  if (ioctl (soc, SIOCOUTQ, &cur_so_sendbuf) == -1)
+    {
+      g_warning ("%s: ioctl error: %s", __func__, strerror (errno));
+      *err = -1;
+      return 0;
+    }
+  return cur_so_sendbuf ? 0 : 1;
+}
+
+/**
+ * @brief Wait until socket send buffer empty or timeout reached.
+ *
+ * @param soc     Socket.
+ * @param timeout Timeout in seconds.
+ */
+void
+wait_until_so_sndbuf_empty (int soc, int timeout)
+{
+  int cnt = 0;
+  int err = 0;
+  int empty;
+
+  empty = so_sndbuf_empty (soc, &err);
+  for (; !empty && (err != -1) && (cnt / 10 != timeout);
+       empty = so_sndbuf_empty (soc, &err), cnt++)
+    {
+      usleep (100000);
+    }
+}
