@@ -1,4 +1,4 @@
-/* Copyright (C) 2014-2019 Greenbone Networks GmbH
+/* Copyright (C) 2014-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -41,7 +41,7 @@
 /**
  * @brief GLib log domain.
  */
-#define G_LOG_DOMAIN "lib  osp"
+#define G_LOG_DOMAIN "libgvm osp"
 
 /**
  * @brief Struct holding options for OSP connection.
@@ -83,12 +83,19 @@ struct osp_credential
  */
 struct osp_target
 {
-  GSList *credentials;      /** Credentials to use in the scan */
-  gchar *exclude_hosts;     /** String defining one or many hosts to exclude */
-  gchar *hosts;             /** String defining one or many hosts to scan */
-  gchar *ports;             /** String defining the ports to scan */
-  gchar *finished_hosts;    /** String defining hosts to exclude as finished */
-  int alive_test;           /** Value defining an alive test method */
+  GSList *credentials;   /** Credentials to use in the scan */
+  gchar *exclude_hosts;  /** String defining one or many hosts to exclude */
+  gchar *hosts;          /** String defining one or many hosts to scan */
+  gchar *ports;          /** String defining the ports to scan */
+  gchar *finished_hosts; /** String defining hosts to exclude as finished */
+  /* Alive test methods can be specified either as bitfield or via selection of
+  individual methods */
+  int alive_test; /** Value defining an alive test method */
+  gboolean icmp;
+  gboolean tcp_syn;
+  gboolean tcp_ack;
+  gboolean arp;
+  gboolean consider_alive;
   int reverse_lookup_unify; /** Value defining reverse_lookup_unify opt */
   int reverse_lookup_only;  /** Value defining reverse_lookup_only opt */
 };
@@ -139,7 +146,10 @@ osp_connection_new (const char *host, int port, const char *cacert,
       connection = g_malloc0 (sizeof (*connection));
       connection->socket = socket (AF_UNIX, SOCK_STREAM, 0);
       if (connection->socket == -1)
-        return NULL;
+        {
+          g_free (connection);
+          return NULL;
+        }
 
       addr.sun_family = AF_UNIX;
       strncpy (addr.sun_path, host, sizeof (addr.sun_path) - 1);
@@ -147,6 +157,7 @@ osp_connection_new (const char *host, int port, const char *cacert,
       if (connect (connection->socket, (struct sockaddr *) &addr, len) == -1)
         {
           close (connection->socket);
+          g_free (connection);
           return NULL;
         }
     }
@@ -887,9 +898,28 @@ target_append_as_xml (osp_target_t *target, GString *xml_string)
                      target->finished_hosts ? target->finished_hosts : "",
                      target->ports ? target->ports : "");
 
+  /* Alive test specified as bitfield */
   if (target->alive_test > 0)
     xml_string_append (xml_string, "<alive_test>%d</alive_test>",
                        target->alive_test);
+  /* Alive test specified via dedicated methods. Dedicted methods are ignored if
+   * alive test was already specified as bitfield.*/
+  else if (target->icmp == TRUE || target->tcp_syn == TRUE
+           || target->tcp_ack == TRUE || target->arp == TRUE
+           || target->consider_alive == TRUE)
+    {
+      xml_string_append (xml_string,
+                         "<alive_test_methods>"
+                         "<icmp>%d</icmp>"
+                         "<tcp_syn>%d</tcp_syn>"
+                         "<tcp_ack>%d</tcp_ack>"
+                         "<arp>%d</arp>"
+                         "<consider_alive>%d</consider_alive>"
+                         "</alive_test_methods>",
+                         target->icmp, target->tcp_syn, target->tcp_ack,
+                         target->arp, target->consider_alive);
+    }
+
   if (target->reverse_lookup_unify == 1)
     xml_string_append (xml_string,
                        "<reverse_lookup_unify>%d</reverse_lookup_unify>",
@@ -1384,7 +1414,7 @@ osp_credential_set_auth_data (osp_credential_t *credential, const char *name,
         g_hash_table_replace (credential->auth_data, g_strdup (name),
                               g_strdup (value));
       else
-        g_hash_table_remove (credential->auth_data, value);
+        g_hash_table_remove (credential->auth_data, name);
     }
   else
     {
@@ -1454,6 +1484,31 @@ osp_target_free (osp_target_t *target)
   g_free (target);
 }
 
+/**
+ * @brief Add alive test methods to OSP target.
+ *
+ * @param[in]  target           The OSP target to add the methods to.
+ * @param[in]  icmp             Use ICMP ping.
+ * @param[in]  tcp_syn          Use TCP-SYN ping.
+ * @param[in]  tcp_ack          Use TCP-ACK ping.
+ * @param[in]  arp              Use ARP ping.
+ * @param[in]  consider_alive   Consider host to be alive.
+ */
+void
+osp_target_add_alive_test_methods (osp_target_t *target, gboolean icmp,
+                                   gboolean tcp_syn, gboolean tcp_ack,
+                                   gboolean arp, gboolean consider_alive)
+{
+  if (!target)
+    return;
+
+  target->icmp = icmp;
+  target->tcp_syn = tcp_syn;
+  target->tcp_ack = tcp_ack;
+  target->arp = arp;
+  target->consider_alive = consider_alive;
+}
+
 /**
  * @brief Add a credential to an OSP target.
  *
