@@ -1,4 +1,4 @@
-/* Copyright (C) 2014-2019 Greenbone Networks GmbH
+/* Copyright (C) 2014-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -35,7 +35,10 @@
 #include <string.h> /* for strlen, strerror, strncpy, memset */
 
 #undef G_LOG_DOMAIN
-#define G_LOG_DOMAIN "lib  kb"
+/**
+ * @brief GLib logging domain.
+ */
+#define G_LOG_DOMAIN "libgvm util"
 
 /**
  * @file kb.c
@@ -62,7 +65,7 @@ struct kb_redis
   unsigned int max_db; /**< Max # of databases. */
   unsigned int db;     /**< Namespace ID number, 0 if uninitialized. */
   redisContext *rctx;  /**< Redis client context. */
-  char path[0];        /**< Path to the server socket. */
+  char *path;          /**< Path to the server socket. */
 };
 #define redis_kb(__kb) ((struct kb_redis *) (__kb))
 
@@ -352,6 +355,7 @@ redis_delete (kb_t kb)
 
   if (kbr->rctx != NULL)
     {
+      g_free (kbr->path);
       redisFree (kbr->rctx);
       kbr->rctx = NULL;
     }
@@ -377,6 +381,33 @@ redis_get_kb_index (kb_t kb)
   return -1;
 }
 
+/**
+ * @brief Attempt to purge dirty pages.
+ *
+ * Attempt to purge dirty pages so these can be reclaimed by the allocator.
+ * This command only works when using jemalloc as an allocator, and evaluates
+ * to a benign NOOP for all others. Command is applied to complete redis
+ * instance and not only single db.
+ *
+ * @param[in] kb KB handle where to run the command.
+ *
+ * @return 0 on success, non-null on error.
+ */
+static int
+redis_memory_purge (kb_t kb)
+{
+  redisReply *rep;
+  int rc = 0;
+
+  rep = redis_cmd (redis_kb (kb), "MEMORY PURGE");
+  if (!rep || rep->type == REDIS_REPLY_ERROR)
+    rc = -1;
+  if (rep)
+    freeReplyObject (rep);
+
+  return rc;
+}
+
 /**
  * @brief Initialize a new Knowledge Base object.
  *
@@ -391,12 +422,15 @@ redis_new (kb_t *kb, const char *kb_path)
   struct kb_redis *kbr;
   int rc = 0;
 
-  kbr = g_malloc0 (sizeof (struct kb_redis) + strlen (kb_path) + 1);
+  kbr = g_malloc0 (sizeof (struct kb_redis));
   kbr->kb.kb_ops = &KBRedisOperations;
-  strncpy (kbr->path, kb_path, strlen (kb_path));
+  kbr->path = g_strdup (kb_path);
 
   if ((rc = get_redis_ctx (kbr)) < 0)
-    return rc;
+    {
+      redis_delete ((kb_t) kbr);
+      return rc;
+    }
   if (redis_test_connection (kbr))
     {
       g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL,
@@ -406,7 +440,15 @@ redis_new (kb_t *kb, const char *kb_path)
       rc = -1;
     }
 
+  /* Ensure that the new kb is clean */
+  redis_delete_all (kbr);
+
   *kb = (kb_t) kbr;
+
+  /* Try to make unused memory available for the OS again. */
+  if (redis_memory_purge (*kb))
+    g_warning ("%s: Memory purge was not successful", __func__);
+
   return rc;
 }
 
@@ -424,9 +466,9 @@ redis_direct_conn (const char *kb_path, const int kb_index)
   struct kb_redis *kbr;
   redisReply *rep;
 
-  kbr = g_malloc0 (sizeof (struct kb_redis) + strlen (kb_path) + 1);
+  kbr = g_malloc0 (sizeof (struct kb_redis));
   kbr->kb.kb_ops = &KBRedisOperations;
-  strncpy (kbr->path, kb_path, strlen (kb_path));
+  kbr->path = g_strdup (kb_path);
 
   kbr->rctx = redisConnectUnix (kbr->path);
   if (kbr->rctx == NULL || kbr->rctx->err)
@@ -435,6 +477,7 @@ redis_direct_conn (const char *kb_path, const int kb_index)
              "%s: redis connection error to %s: %s", __func__, kbr->path,
              kbr->rctx ? kbr->rctx->errstr : strerror (ENOMEM));
       redisFree (kbr->rctx);
+      g_free (kbr->path);
       g_free (kbr);
       return NULL;
     }
@@ -446,6 +489,8 @@ redis_direct_conn (const char *kb_path, const int kb_index)
         freeReplyObject (rep);
       redisFree (kbr->rctx);
       kbr->rctx = NULL;
+      g_free (kbr->path);
+      g_free (kbr);
       return NULL;
     }
   freeReplyObject (rep);
@@ -466,9 +511,9 @@ redis_find (const char *kb_path, const char *key)
   struct kb_redis *kbr;
   unsigned int i = 1;
 
-  kbr = g_malloc0 (sizeof (struct kb_redis) + strlen (kb_path) + 1);
+  kbr = g_malloc0 (sizeof (struct kb_redis));
   kbr->kb.kb_ops = &KBRedisOperations;
-  strncpy (kbr->path, kb_path, strlen (kb_path));
+  kbr->path = g_strdup (kb_path);
 
   do
     {
@@ -481,6 +526,7 @@ redis_find (const char *kb_path, const char *key)
                  "%s: redis connection error to %s: %s", __func__, kbr->path,
                  kbr->rctx ? kbr->rctx->errstr : strerror (ENOMEM));
           redisFree (kbr->rctx);
+          g_free (kbr->path);
           g_free (kbr);
           return NULL;
         }
@@ -524,6 +570,7 @@ redis_find (const char *kb_path, const char *key)
     }
   while (i < kbr->max_db);
 
+  g_free (kbr->path);
   g_free (kbr);
   return NULL;
 }
@@ -1112,11 +1159,14 @@ redis_del_items (kb_t kb, const char *name)
  * @param[in] name  Item name.
  * @param[in] str  Item value.
  * @param[in] len  Value length. Used for blobs.
+ * @param[in] pos  Which position the value is appended to. 0 for right,
+ *                 1 for left position in the list.
  *
  * @return 0 on success, non-null on error.
  */
 static int
-redis_add_str_unique (kb_t kb, const char *name, const char *str, size_t len)
+redis_add_str_unique (kb_t kb, const char *name, const char *str, size_t len,
+                      int pos)
 {
   struct kb_redis *kbr;
   redisReply *rep = NULL;
@@ -1135,7 +1185,7 @@ redis_add_str_unique (kb_t kb, const char *name, const char *str, size_t len)
   if (len == 0)
     {
       redisAppendCommand (ctx, "LREM %s 1 %s", name, str);
-      redisAppendCommand (ctx, "RPUSH %s %s", name, str);
+      redisAppendCommand (ctx, "%s %s %s", pos ? "LPUSH" : "RPUSH", name, str);
       redisGetReply (ctx, (void **) &rep);
       if (rep && rep->type == REDIS_REPLY_INTEGER && rep->integer == 1)
         g_debug ("Key '%s' already contained value '%s'", name, str);
@@ -1145,7 +1195,8 @@ redis_add_str_unique (kb_t kb, const char *name, const char *str, size_t len)
   else
     {
       redisAppendCommand (ctx, "LREM %s 1 %b", name, str, len);
-      redisAppendCommand (ctx, "RPUSH %s %b", name, str, len);
+      redisAppendCommand (ctx, "%s %s %b", pos ? "LPUSH" : "RPUSH", name, str,
+                          len);
       redisGetReply (ctx, (void **) &rep);
       if (rep && rep->type == REDIS_REPLY_INTEGER && rep->integer == 1)
         g_debug ("Key '%s' already contained string '%s'", name, str);
@@ -1497,6 +1548,7 @@ redis_flush_all (kb_t kb, const char *except)
     }
   while (i < kbr->max_db);
 
+  g_free (kbr->path);
   g_free (kb);
   return 0;
 }
@@ -1555,7 +1607,8 @@ redis_delete_all (struct kb_redis *kbr)
   if (sigaction (SIGPIPE, &new_action, &original_action))
     return -1;
 
-  g_debug ("%s: deleting all elements from KB #%u", __func__, kbr->db);
+  if (kbr)
+    g_debug ("%s: deleting all elements from KB #%u", __func__, kbr->db);
   rep = redis_cmd (kbr, "FLUSHDB");
   if (rep == NULL || rep->type != REDIS_REPLY_STATUS)
     {
