@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2019 Greenbone Networks GmbH
+/* Copyright (C) 2009-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -42,9 +42,9 @@
 
 #undef G_LOG_DOMAIN
 /**
- * @brief GLib log domain.
+ * @brief GLib logging domain.
  */
-#define G_LOG_DOMAIN "lib  nvticache"
+#define G_LOG_DOMAIN "libgvm util"
 
 char *src_path = NULL; /**< The directory of the source files. */
 kb_t cache_kb = NULL;  /**< Cache KB handler. */
