@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2019 Greenbone Networks GmbH
+/* Copyright (C) 2015-2021 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -46,6 +46,12 @@
 
 #include <glib.h> /* for g_warning */
 
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib logging domain.
+ */
+#define G_LOG_DOMAIN "libgvm util"
+
 #ifndef PW_MAX_MSG_SIZE
 #define PW_MAX_MSG_SIZE 4096
 #endif
